function [DxDx,DyDy,DxDy,DxM,DyM,MM]=matriceP2(v1,t1,ne1)
%
% Calcul des matrices de rigidité (DxDx,DyDy,DxDy) en P2,
% de la matrice de la divergence (DxM, DyM) en P2/P1 
% et de la matrice de masse MM en P2
%
    nv1=size(v1,1);
    nt1=size(t1,1);
    nv2=nv1+ne1;
    MM=sparse(nv2,nv2);
    DxDx=sparse(nv2,nv2); DyDy=DxDx; DxDy=DxDx;
    DxM=sparse(nv2,nv1);DyM=DxM;
    
    %Matrice de masse élémentaire P2/P2
    MM_elem=[ 6 -1 -1 -4  0  0;
             -1  6 -1  0 -4  0;
             -1 -1  6  0  0 -4;
             -4  0  0 32 16 16;
              0 -4  0 16 32 16;
              0  0 -4 16 16 32]/180;

    %Matrice de rigidité élémentaire P2/P2

    DxDx_elem=[ 3  1  0  0  0 -4;
                1  3  0  0  0 -4;
                0  0  0  0  0  0;
                0  0  0  8 -8  0;
                0  0  0 -8  8  0;
               -4 -4  0  0  0  8]/6;


    DyDy_elem=[ 3  0  1  0 -4  0;
                0  0  0  0  0  0;
                1  0  3  0 -4  0;
                0  0  0  8  0 -8;
               -4  0 -4  0  8  0;
                0  0  0 -8  0  8]/6;

    DxDy_elem=[ 3  0  1  0 -4  0;
                1  0 -1  4  0 -4;
                0  0  0  0  0  0;
                0  0  4  4 -4 -4;
                0  0 -4 -4  4  4;
               -4  0  0 -4  4  4]/6;

    %Matrice de divergence élémentaire P2/P1

    DxM_elem=[-1  0  0;
               0  1  0;
               0  0  0;
               1  1  2;
              -1 -1 -2;
               1 -1  0]/6;

    DyM_elem=[-1  0  0;
               0  0  0;
               0  0  1;
               1  2  1;
               1  0 -1;
              -1 -2 -1]/6;

    cmpt=1;
    fprintf('(1...20)=');
    ntt=floor(nt1/20)*20;

    for k=1:nt1
        if(mod((k*20),ntt)==0)
            fprintf(strcat(num2str(cmpt),' '));
            cmpt=cmpt+1;
        end
        i1=t1(k,1:3);
        i2=[i1 t1(k,6:8)+nv1];
        p=v1(i1,1:2);
        aire=t1(k,5);

        %matrice de masse P2/P2
        MM(i2,i2)=MM(i2,i2)+aire*MM_elem;

        %matrice de rigidite DxDx
        DxDxk= (p(3,2)-p(1,2))^2*DxDx_elem...
            -(p(3,2)-p(1,2))*(p(2,2)-p(1,2))*(DxDy_elem+DxDy_elem')...
            +(p(2,2)-p(1,2))^2*DyDy_elem;

        DxDx(i2,i2) = DxDx(i2,i2) + DxDxk/(2*aire);

        %matrice de rigidite DyDy
        DyDyk= (p(3,1)-p(1,1))^2*DxDx_elem...
            -(p(3,1)-p(1,1))*(p(2,1)-p(1,1))*(DxDy_elem+DxDy_elem')...
            +(p(2,1)-p(1,1))^2*DyDy_elem;

        DyDy(i2,i2) = DyDy(i2,i2) + DyDyk/(2*aire);

        %matrice de rigidite DxDy
        DxDyk=-(p(3,1)-p(1,1))*(p(3,2)-p(1,2))*DxDx_elem...
            +(p(2,1)-p(1,1))*(p(3,2)-p(1,2))*DxDy_elem...
            +(p(2,2)-p(1,2))*(p(3,1)-p(1,1))*DxDy_elem'...
            -(p(2,1)-p(1,1))*(p(2,2)-p(1,2))*DyDy_elem;

        DxDy(i2,i2) = DxDy(i2,i2) + DxDyk/(2*aire);

        %Matrice de divergence DxM
        signe=sign((p(2,1)-p(1,1))*(p(3,2)-p(1,2))...
                  -(p(3,1)-p(1,1))*(p(2,2)-p(1,2)));
        DxMk=(p(3,2)-p(1,2))*DxM_elem-(p(2,2)-p(1,2))*DyM_elem;
        DxM(i2,i1)=DxM(i2,i1) +signe*DxMk;

        %matrice de divergence DyM
        DyMk= -(p(3,1)-p(1,1))*DxM_elem+(p(2,1)-p(1,1))*DyM_elem;
        DyM(i2,i1)=DyM(i2,i1) +signe*DyMk;

    end
    fprintf('\n');
end