function [DxDx,DyDy,DxDy,DxM,DyM,MM]=matriceP2vect(v1,t1)
%
% Calcul des matrices de rigidité (DxDx,DyDy,DxDy) en P2,
% de la matrice de la divergence (DxM, DyM) en P2/P1 
% et de la matrice de masse MM en P2
%
% Version "vectorisée" (bcp plus rapide: facteur > 10...) de MatriceP2.m : 
% on travaille avec des vecteurs à la place des matrices (construction des 
% matrices tout à la fin.
%
    nv1=size(v1,1);
    nt1=size(t1,1);

    %Matrice de masse elementaire P2/P2
    matMM_elem=[ 6 -1 -1 -4  0  0;
                -1  6 -1  0 -4  0;
                -1 -1  6  0  0 -4;
                -4  0  0 32 16 16;
                 0 -4  0 16 32 16;
                 0  0 -4 16 16 32]/180;

    MM_elem = reshape(matMM_elem',1,36);

    %Matrices de rigidité élémentaires P2/P2
    matDxDx_elem=[ 3  1  0  0  0 -4;
                   1  3  0  0  0 -4;
                   0  0  0  0  0  0;
                   0  0  0  8 -8  0;
                   0  0  0 -8  8  0;
                  -4 -4  0  0  0  8]/6;

    DxDx_elem = reshape(matDxDx_elem',1,36);

    matDyDy_elem=[ 3  0  1  0 -4  0;
                   0  0  0  0  0  0;
                   1  0  3  0 -4  0;
                   0  0  0  8  0 -8;
                  -4  0 -4  0  8  0;
                   0  0  0 -8  0  8]/6;

    DyDy_elem = reshape(matDyDy_elem',1,36);

    matDxDy_elem=[ 3  0  1  0 -4  0;
                   1  0 -1  4  0 -4;
                   0  0  0  0  0  0;
                   0  0  4  4 -4 -4;
                   0  0 -4 -4  4  4;
                  -4  0  0 -4  4  4]/6;

    DxDy_elem = reshape(matDxDy_elem',1,36);
    DxDy_elem_transp = reshape(matDxDy_elem,1,36); 

    %Matrices de divergence élémentaires P2/P1
    matDxM_elem=[-1  0  0;
                  0  1  0;
                  0  0  0;
                  1  1  2;
                 -1 -1 -2;
                  1 -1  0]/6;

    DxM_elem = reshape(matDxM_elem',1,18);        

    matDyM_elem=[-1  0  0;
                  0  0  0;
                  0  0  1;
                  1  2  1;
                  1  0 -1;
                 -1 -2 -1]/6;

    DyM_elem = reshape(matDyM_elem',1,18);        

    cmpt=1;
    fprintf('(1...20)=');
    ntt=floor(nt1/20)*20;

    indMMi=zeros(nt1*36,1);
    indMMj=zeros(nt1*36,1);
    MMval=zeros(nt1*36,1);
    DxDxval=zeros(nt1*36,1);
    DyDyval=zeros(nt1*36,1);
    DxDyval=zeros(nt1*36,1);

    indDxMi=zeros(nt1*18,1);
    indDxMj=zeros(nt1*18,1);
    DxMval=zeros(nt1*18,1);
    DyMval=zeros(nt1*18,1);

    ind_ligne1=1;
    ind_ligne2=1;

    for k=1:nt1
        if(mod((k*20),ntt)==0)
            fprintf(strcat(num2str(cmpt),' '));
            cmpt=cmpt+1;
        end
        i1=t1(k,1:3);
        i2=[i1 t1(k,6:8)+nv1];
        p=v1(i1,1:2);
        aire=t1(k,5);

        %Vecteur de masse P2/P2
        MMval(ind_ligne1:ind_ligne1+35)=aire*MM_elem';

        indice=ones(6,1)*i2;
        indMMi(ind_ligne1:ind_ligne1+35)=indice(:);
        indice=indice';
        indMMj(ind_ligne1:ind_ligne1+35)=indice(:);

        %Vecteur de rigidite DxDx
        DxDxk= (p(3,2)-p(1,2))^2*DxDx_elem...
           -(p(3,2)-p(1,2))*(p(2,2)-p(1,2))*(DxDy_elem+DxDy_elem_transp)...
           +(p(2,2)-p(1,2))^2*DyDy_elem;
        DxDxval(ind_ligne1:ind_ligne1+35)=DxDxk'/(2*aire);

        %Vecteur de rigidite DyDy
        DyDyk= (p(3,1)-p(1,1))^2*DxDx_elem...
           -(p(3,1)-p(1,1))*(p(2,1)-p(1,1))*(DxDy_elem+DxDy_elem_transp)...
           +(p(2,1)-p(1,1))^2*DyDy_elem;
        DyDyval(ind_ligne1:ind_ligne1+35)=DyDyk'/(2*aire);

        %Vecteur de rigidite DxDy
        DxDyk=-(p(3,1)-p(1,1))*(p(3,2)-p(1,2))*DxDx_elem...
            +(p(2,1)-p(1,1))*(p(3,2)-p(1,2))*DxDy_elem...
            +(p(2,2)-p(1,2))*(p(3,1)-p(1,1))*DxDy_elem_transp...
            -(p(2,1)-p(1,1))*(p(2,2)-p(1,2))*DyDy_elem;
        DxDyval(ind_ligne1:ind_ligne1+35)=DxDyk'/(2*aire);

        %Vecteur de divergence DxM
        signe=sign((p(2,1)-p(1,1))*(p(3,2)-p(1,2))...
                  -(p(3,1)-p(1,1))*(p(2,2)-p(1,2)));
        DxMk=(p(3,2)-p(1,2))*DxM_elem-(p(2,2)-p(1,2))*DyM_elem;
        DxMval(ind_ligne2:ind_ligne2+17)=signe*DxMk';

        %Vecteur de divergence DyM
        DyMk= -(p(3,1)-p(1,1))*DxM_elem+(p(2,1)-p(1,1))*DyM_elem;
        DyMval(ind_ligne2:ind_ligne2+17)=signe*DyMk';

        indice=ones(3,1)*i2;
        indDxMi(ind_ligne2:ind_ligne2+17)=indice(:);
        indice=i1'*ones(1,6);
        indDxMj(ind_ligne2:ind_ligne2+17)=indice(:);

        ind_ligne1=ind_ligne1+36;
        ind_ligne2=ind_ligne2+18;
    end

    % Construction des matrices
    MM=sparse(indMMi,indMMj,MMval);
    DxDx=sparse(indMMi,indMMj,DxDxval);
    DyDy=sparse(indMMi,indMMj,DyDyval);
    DxDy=sparse(indMMi,indMMj,DxDyval);
    DxM=sparse(indDxMi,indDxMj,DxMval);
    DyM=sparse(indDxMi,indDxMj,DyMval);

    fprintf('\n');
end
